@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """ This is the main actions of Nuitka.
@@ -35,12 +35,19 @@ from nuitka.freezer.IncludedEntryPoints import (
     getStandaloneEntryPoints,
     setMainEntryPoint,
 )
-from nuitka.importing import Importing, Recursion
+from nuitka.importing.Importing import locateModule, setupImportingFromOptions
+from nuitka.importing.Recursion import (
+    scanIncludedPackage,
+    scanPluginFilenamePattern,
+    scanPluginPath,
+    scanPluginSinglePath,
+)
 from nuitka.Options import (
     getPythonPgoInput,
     hasPythonFlagIsolated,
     hasPythonFlagNoAnnotations,
     hasPythonFlagNoAsserts,
+    hasPythonFlagNoBytecodeRuntimeCache,
     hasPythonFlagNoDocStrings,
     hasPythonFlagNoWarnings,
     hasPythonFlagUnbuffered,
@@ -54,6 +61,7 @@ from nuitka.Progress import (
     setupProgressBar,
 )
 from nuitka.PythonFlavors import (
+    getPythonFlavorName,
     isApplePython,
     isArchPackagePython,
     isDebianPackagePython,
@@ -69,7 +77,7 @@ from nuitka.PythonVersions import (
     python_version_str,
 )
 from nuitka.Serialization import ConstantAccessor
-from nuitka.Tracing import general, inclusion_logger
+from nuitka.Tracing import general, inclusion_logger, pgo_logger
 from nuitka.tree import SyntaxErrors
 from nuitka.tree.ReformulationMultidist import createMultidistMainSourceCode
 from nuitka.utils import InstanceCounters
@@ -83,6 +91,7 @@ from nuitka.utils.FileOperations import (
     changeFilenameExtension,
     deleteFile,
     getExternalUsePath,
+    getReportPath,
     openTextFile,
     removeDirectory,
     resetDirectory,
@@ -102,8 +111,9 @@ from . import ModuleRegistry, Options, OutputDirectories
 from .build.SconsInterface import (
     asBoolStr,
     cleanSconsDirectory,
+    getCommonSconsOptions,
     runScons,
-    setCommonSconsOptions,
+    setPythonTargetOptions,
 )
 from .code_generation import CodeGeneration, LoaderCodes, Reports
 from .finalizations import Finalization
@@ -130,24 +140,19 @@ def _createMainModule():
     directory paths.
 
     """
-    # Many cases and details to deal with, pylint: disable=too-many-branches,too-many-locals
+    # Many cases and details to deal with, pylint: disable=too-many-branches
 
     Plugins.onBeforeCodeParsing()
 
-    main_filenames = Options.getMainEntryPointFilenames()
-
     # First, build the raw node tree from the source code.
-    if len(main_filenames) > 1:
+    if Options.isMultidistMode():
         assert not Options.shallMakeModule()
 
         main_module = buildMainModuleTree(
-            # TODO: Should not be given.
-            filename=main_filenames[0],
-            source_code=createMultidistMainSourceCode(main_filenames),
+            source_code=createMultidistMainSourceCode(),
         )
     else:
         main_module = buildMainModuleTree(
-            filename=main_filenames[0],
             source_code=None,
         )
 
@@ -162,7 +167,11 @@ def _createMainModule():
                 % distribution_name
             )
 
-        addDistributionMetadataValue(distribution_name, distribution)
+        addDistributionMetadataValue(
+            distribution_name=distribution_name,
+            distribution=distribution,
+            reason="user requested",
+        )
 
     # First remove old object files and old generated files, old binary or
     # module, and standalone mode program directory if any, they can only do
@@ -175,11 +184,19 @@ def _createMainModule():
     # Prepare the ".dist" directory, throwing away what was there before.
     if Options.isStandaloneMode():
         standalone_dir = OutputDirectories.getStandaloneDirectoryPath(bundle=False)
-        resetDirectory(path=standalone_dir, ignore_errors=True)
+        resetDirectory(
+            path=standalone_dir,
+            logger=general,
+            ignore_errors=True,
+            extra_recommendation="Stop previous binary.",
+        )
 
         if Options.shallCreateAppBundle():
             resetDirectory(
-                path=changeFilenameExtension(standalone_dir, ".app"), ignore_errors=True
+                path=changeFilenameExtension(standalone_dir, ".app"),
+                logger=general,
+                ignore_errors=True,
+                extra_recommendation=None,
             )
 
     # Delete result file, to avoid confusion with previous build and to
@@ -197,31 +214,20 @@ def _createMainModule():
 
     # Second, do it for the directories given.
     for plugin_filename in Options.getShallFollowExtra():
-        Recursion.checkPluginPath(plugin_filename=plugin_filename, module_package=None)
+        scanPluginPath(plugin_filename=plugin_filename, module_package=None)
 
     for pattern in Options.getShallFollowExtraFilePatterns():
-        Recursion.checkPluginFilenamePattern(pattern=pattern)
+        scanPluginFilenamePattern(pattern=pattern)
+
+    # For packages, include the full suite.
+    if Options.shallMakePackage():
+        scanIncludedPackage(main_module.getFullName())
 
     for package_name in Options.getMustIncludePackages():
-        package_name, package_directory, _module_kind, finding = Importing.locateModule(
-            module_name=ModuleName(package_name),
-            parent_package=None,
-            level=0,
-        )
-
-        if finding != "absolute":
-            inclusion_logger.sysexit(
-                "Error, failed to locate package '%s' you asked to include."
-                % package_name
-            )
-
-        Recursion.checkPluginPath(
-            plugin_filename=package_directory,
-            module_package=package_name.getPackageName(),
-        )
+        scanIncludedPackage(package_name)
 
     for module_name in Options.getMustIncludeModules():
-        module_name, module_filename, _module_kind, finding = Importing.locateModule(
+        module_name, module_filename, _module_kind, finding = locateModule(
             module_name=ModuleName(module_name),
             parent_package=None,
             level=0,
@@ -233,7 +239,7 @@ def _createMainModule():
                 % module_name.asString()
             )
 
-        Recursion.checkPluginSinglePath(
+        scanPluginSinglePath(
             plugin_filename=module_filename,
             module_package=module_name.getPackageName(),
             package_only=True,
@@ -253,15 +259,11 @@ def _createMainModule():
         checkFreezingModuleSet()
 
     # Check if distribution meta data is included, that cannot be used.
-    for distribution_name, (
-        package_name,
-        _metadata,
-        _entry_points,
-    ) in getDistributionMetadataValues():
-        if not ModuleRegistry.hasDoneModule(package_name):
+    for distribution_name, meta_data_value in getDistributionMetadataValues():
+        if not ModuleRegistry.hasDoneModule(meta_data_value.module_name):
             inclusion_logger.sysexit(
                 "Error, including metadata for distribution '%s' without including related package '%s'."
-                % (distribution_name, package_name)
+                % (distribution_name, meta_data_value.module_name)
             )
 
     # Allow plugins to comment on final module set.
@@ -526,11 +528,11 @@ def _runCPgoBinary():
                 source_dir=OutputDirectories.getSourceDirectoryPath(), key="PATH"
             ),
         ):
-            _exit_code = _runPgoBinary()
+            exit_code_pgo = _runPgoBinary()
 
         pgo_data_collected = os.path.exists(msvc_pgc_filename)
     else:
-        _exit_code = _runPgoBinary()
+        exit_code_pgo = _runPgoBinary()
 
         # gcc file suffix, spell-checker: ignore gcda
         gcc_constants_pgo_filename = os.path.join(
@@ -539,12 +541,21 @@ def _runCPgoBinary():
 
         pgo_data_collected = os.path.exists(gcc_constants_pgo_filename)
 
-    if not pgo_data_collected:
-        general.sysexit(
-            "Error, no PGO information produced, did the created binary run at all?"
+    if exit_code_pgo != 0:
+        pgo_logger.warning(
+            """\
+Error, the C PGO compiled program error exited. Make sure it works \
+fully before using '--pgo-c' option."""
         )
 
-    general.info("Successfully collected C level PGO information.", style="blue")
+    if not pgo_data_collected:
+        pgo_logger.sysexit(
+            """\
+Error, no C PGO compiled program did not produce expected information, \
+did the created binary run at all?"""
+        )
+
+    pgo_logger.info("Successfully collected C level PGO information.", style="blue")
 
 
 def _runPythonPgoBinary():
@@ -559,7 +570,9 @@ def _runPythonPgoBinary():
 
     if not os.path.exists(pgo_filename):
         general.sysexit(
-            "Error, no Python PGO information produced, did the created binary run (exit code %d) as expected?"
+            """\
+Error, no Python PGO information produced, did the created binary
+run (exit code %d) as expected?"""
             % exit_code
         )
 
@@ -570,194 +583,198 @@ def runSconsBackend():
     # Scons gets transported many details, that we express as variables, and
     # have checks for them, leading to many branches and statements,
     # pylint: disable=too-many-branches,too-many-statements
+    scons_options, env_values = getCommonSconsOptions()
 
-    options = {
-        "source_dir": OutputDirectories.getSourceDirectoryPath(),
-        "nuitka_python": asBoolStr(isNuitkaPython()),
-        "debug_mode": asBoolStr(Options.is_debug),
-        "debugger_mode": asBoolStr(Options.shallRunInDebugger()),
-        "python_debug": asBoolStr(Options.shallUsePythonDebug()),
-        "module_mode": asBoolStr(Options.shallMakeModule()),
-        "full_compat": asBoolStr(Options.is_full_compat),
-        "experimental": ",".join(Options.getExperimentalIndications()),
-        "trace_mode": asBoolStr(Options.shallTraceExecution()),
-        "file_reference_mode": Options.getFileReferenceMode(),
-        "module_count": "%d" % len(ModuleRegistry.getDoneModules()),
-    }
+    setPythonTargetOptions(scons_options)
+
+    scons_options["source_dir"] = OutputDirectories.getSourceDirectoryPath()
+    scons_options["nuitka_python"] = asBoolStr(isNuitkaPython())
+    scons_options["debug_mode"] = asBoolStr(Options.is_debug)
+    scons_options["debugger_mode"] = asBoolStr(Options.shallRunInDebugger())
+    scons_options["python_debug"] = asBoolStr(Options.shallUsePythonDebug())
+    scons_options["module_mode"] = asBoolStr(Options.shallMakeModule())
+    scons_options["full_compat"] = asBoolStr(Options.is_full_compat)
+    scons_options["experimental"] = ",".join(Options.getExperimentalIndications())
+    scons_options["trace_mode"] = asBoolStr(Options.shallTraceExecution())
+    scons_options["file_reference_mode"] = Options.getFileReferenceMode()
+    scons_options["module_count"] = "%d" % len(ModuleRegistry.getDoneModules())
 
     if Options.isLowMemory():
-        options["low_memory"] = asBoolStr(True)
+        scons_options["low_memory"] = asBoolStr(True)
 
     if not Options.shallMakeModule():
-        options["result_exe"] = OutputDirectories.getResultFullpath(onefile=False)
+        scons_options["result_exe"] = OutputDirectories.getResultFullpath(onefile=False)
 
         main_module = ModuleRegistry.getRootTopModule()
         assert main_module.isMainModule()
 
         main_module_name = main_module.getFullName()
         if main_module_name != "__main__":
-            options["main_module_name"] = main_module_name
+            scons_options["main_module_name"] = main_module_name
 
     if Options.shallUseStaticLibPython():
-        options["static_libpython"] = getSystemStaticLibPythonPath()
+        scons_options["static_libpython"] = getSystemStaticLibPythonPath()
 
     if isDebianPackagePython():
-        options["debian_python"] = asBoolStr(True)
+        scons_options["debian_python"] = asBoolStr(True)
     if isFedoraPackagePython():
-        options["fedora_python"] = asBoolStr(True)
+        scons_options["fedora_python"] = asBoolStr(True)
     if isArchPackagePython():
-        options["arch_python"] = asBoolStr(True)
+        scons_options["arch_python"] = asBoolStr(True)
     if isApplePython():
-        options["apple_python"] = asBoolStr(True)
+        scons_options["apple_python"] = asBoolStr(True)
     if isPyenvPython():
-        options["pyenv_python"] = asBoolStr(True)
+        scons_options["pyenv_python"] = asBoolStr(True)
 
     if Options.isStandaloneMode():
-        options["standalone_mode"] = asBoolStr(True)
+        scons_options["standalone_mode"] = asBoolStr(True)
 
     if Options.isOnefileMode():
-        options["onefile_mode"] = asBoolStr(True)
+        scons_options["onefile_mode"] = asBoolStr(True)
 
         if Options.isOnefileTempDirMode():
-            options["onefile_temp_mode"] = asBoolStr(True)
+            scons_options["onefile_temp_mode"] = asBoolStr(True)
 
+    # TODO: Some things are going to hate that, we might need to bundle
+    # for accelerated mode still.
     if Options.shallCreateAppBundle():
-        options["macos_bundle_mode"] = asBoolStr(True)
+        scons_options["macos_bundle_mode"] = asBoolStr(True)
 
     if Options.getForcedStdoutPath():
-        options["forced_stdout_path"] = Options.getForcedStdoutPath()
+        scons_options["forced_stdout_path"] = Options.getForcedStdoutPath()
 
     if Options.getForcedStderrPath():
-        options["forced_stderr_path"] = Options.getForcedStderrPath()
+        scons_options["forced_stderr_path"] = Options.getForcedStderrPath()
+
+    if Options.isProfile():
+        scons_options["profile_mode"] = asBoolStr(True)
 
     if Options.shallTreatUninstalledPython():
-        options["uninstalled_python"] = asBoolStr(True)
+        scons_options["uninstalled_python"] = asBoolStr(True)
 
     if ModuleRegistry.getUncompiledTechnicalModules():
-        options["frozen_modules"] = str(
+        scons_options["frozen_modules"] = str(
             len(ModuleRegistry.getUncompiledTechnicalModules())
         )
 
-    if Options.isProfile():
-        options["profile_mode"] = asBoolStr(True)
-
     if hasPythonFlagNoWarnings():
-        options["no_python_warnings"] = asBoolStr(True)
+        scons_options["no_python_warnings"] = asBoolStr(True)
 
     if hasPythonFlagNoAsserts():
-        options["python_sysflag_optimize"] = str(
+        scons_options["python_sysflag_optimize"] = str(
             2 if hasPythonFlagNoDocStrings() else 1
         )
 
-        options["python_flag_no_asserts"] = asBoolStr(True)
+        scons_options["python_flag_no_asserts"] = asBoolStr(True)
 
     if hasPythonFlagNoDocStrings():
-        options["python_flag_no_docstrings"] = asBoolStr(True)
+        scons_options["python_flag_no_docstrings"] = asBoolStr(True)
 
     if hasPythonFlagNoAnnotations():
-        options["python_flag_no_annotations"] = asBoolStr(True)
+        scons_options["python_flag_no_annotations"] = asBoolStr(True)
 
     if python_version < 0x300 and sys.flags.py3k_warning:
-        options["python_sysflag_py3k_warning"] = asBoolStr(True)
+        scons_options["python_sysflag_py3k_warning"] = asBoolStr(True)
 
     if python_version < 0x300 and (
         sys.flags.division_warning or sys.flags.py3k_warning
     ):
-        options["python_sysflag_division_warning"] = asBoolStr(True)
+        scons_options["python_sysflag_division_warning"] = asBoolStr(True)
 
     if sys.flags.bytes_warning:
-        options["python_sysflag_bytes_warning"] = asBoolStr(True)
+        scons_options["python_sysflag_bytes_warning"] = asBoolStr(True)
 
     if int(os.getenv("NUITKA_NOSITE_FLAG", Options.hasPythonFlagNoSite())):
-        options["python_sysflag_no_site"] = asBoolStr(True)
+        scons_options["python_sysflag_no_site"] = asBoolStr(True)
 
     if Options.hasPythonFlagTraceImports():
-        options["python_sysflag_verbose"] = asBoolStr(True)
+        scons_options["python_sysflag_verbose"] = asBoolStr(True)
 
     if Options.hasPythonFlagNoRandomization():
-        options["python_sysflag_no_randomization"] = asBoolStr(True)
+        scons_options["python_sysflag_no_randomization"] = asBoolStr(True)
 
     if python_version < 0x300 and sys.flags.unicode:
-        options["python_sysflag_unicode"] = asBoolStr(True)
+        scons_options["python_sysflag_unicode"] = asBoolStr(True)
 
     if python_version >= 0x370 and sys.flags.utf8_mode:
-        options["python_sysflag_utf8"] = asBoolStr(True)
+        scons_options["python_sysflag_utf8"] = asBoolStr(True)
+
+    if hasPythonFlagNoBytecodeRuntimeCache():
+        scons_options["python_sysflag_unbuffered"] = asBoolStr(True)
 
     if hasPythonFlagUnbuffered():
-        options["python_sysflag_unbuffered"] = asBoolStr(True)
+        scons_options["python_sysflag_unbuffered"] = asBoolStr(True)
 
     if hasPythonFlagIsolated():
-        options["python_sysflag_isolated"] = asBoolStr(True)
+        scons_options["python_sysflag_isolated"] = asBoolStr(True)
 
     abiflags = getPythonABI()
     if abiflags:
-        options["abiflags"] = abiflags
+        scons_options["abiflags"] = abiflags
 
     if Options.shallMakeModule():
-        options["result_exe"] = OutputDirectories.getResultBasePath(
+        scons_options["result_exe"] = OutputDirectories.getResultBasePath(
             onefile=False
         ) + getSharedLibrarySuffix(preferred=True)
 
     link_module_libs = getModuleLinkerLibs()
     if link_module_libs:
-        options["link_module_libs"] = ",".join(link_module_libs)
-
-    env_values = setCommonSconsOptions(options)
+        scons_options["link_module_libs"] = ",".join(link_module_libs)
 
     # Allow plugins to build definitions.
     env_values.update(Plugins.getBuildDefinitions())
 
-    if Options.shallCreatePgoInput():
-        options["pgo_mode"] = "python"
+    if Options.shallCreatePythonPgoInput():
+        scons_options["pgo_mode"] = "python"
 
         result = runScons(
-            options=options,
+            scons_options=scons_options,
             env_values=env_values,
             scons_filename="Backend.scons",
         )
 
         if not result:
-            return result, options
+            return result, scons_options
 
         # Need to make it usable before executing it.
         executePostProcessing()
         _runPythonPgoBinary()
 
-        return True, options
+        return True, scons_options
 
         # Need to restart compilation from scratch here.
-    if Options.isPgoMode():
+    if Options.isCPgoMode():
         # For C level PGO, we have a 2 pass system. TODO: Make it more global for onefile
         # and standalone mode proper support, which might need data files to be
         # there, which currently are not yet there, so it won't run.
-        if Options.isPgoMode():
-            options["pgo_mode"] = "generate"
+        if Options.isCPgoMode():
+            scons_options["pgo_mode"] = "generate"
 
             result = runScons(
-                options=options,
+                scons_options=scons_options,
                 env_values=env_values,
                 scons_filename="Backend.scons",
             )
 
             if not result:
-                return result, options
+                return result, scons_options
 
             # Need to make it usable before executing it.
             executePostProcessing()
             _runCPgoBinary()
-            options["pgo_mode"] = "use"
+            scons_options["pgo_mode"] = "use"
 
     result = (
         runScons(
-            options=options,
+            scons_options=scons_options,
             env_values=env_values,
             scons_filename="Backend.scons",
         ),
-        options,
+        scons_options,
     )
 
     # Delete PGO files if asked to do that.
-    if options.get("pgo_mode") == "use" and _wasMsvcMode():
+    if scons_options.get("pgo_mode") == "use" and _wasMsvcMode():
         _deleteMsvcPGOFiles(pgo_mode="use")
 
     return result
@@ -778,8 +795,8 @@ def callExecPython(args, add_path, uac):
 
 def _executeMain(binary_filename):
     # Wrap in debugger, unless the CMD file contains that call already.
-    if Options.shallRunInDebugger() and not Options.shallCreateCmdFileForExecution():
-        args = wrapCommandForDebuggerForExec(binary_filename)
+    if Options.shallRunInDebugger() and not Options.shallCreateScriptFileForExecution():
+        args = wrapCommandForDebuggerForExec(command=(binary_filename,))
     else:
         args = (binary_filename, binary_filename)
 
@@ -793,7 +810,7 @@ def _executeMain(binary_filename):
 def _executeModule(tree):
     """Execute the extension module just created."""
 
-    if python_version < 0x340:
+    if python_version < 0x300:
         python_command_template = """\
 import os, imp;\
 assert os.path.normcase(os.path.abspath(os.path.normpath(\
@@ -828,7 +845,9 @@ import sys; sys.path.insert(0, %(output_dir)r)
     }
 
     if Options.shallRunInDebugger():
-        args = wrapCommandForDebuggerForExec(sys.executable, "-c", python_command)
+        args = wrapCommandForDebuggerForExec(
+            command=(sys.executable, "-c", python_command)
+        )
     else:
         args = (sys.executable, "python", "-c", python_command)
 
@@ -885,8 +904,7 @@ def compileTree():
     if Options.isShowMemory():
         InstanceCounters.printStats()
 
-    if Options.is_debug:
-        Reports.doMissingOptimizationReport()
+    Reports.doMissingOptimizationReport()
 
     if Options.shallNotDoExecCCompilerCall():
         return True, {}
@@ -950,9 +968,10 @@ def _main():
         readPGOInputFile(pgo_filename)
 
     general.info(
-        "Starting Python compilation with Nuitka %r on Python %r commercial grade %r."
+        "Starting Python compilation with Nuitka %r on Python (flavor %s), %r commercial grade %r."
         % (
             getNuitkaVersion(),
+            getPythonFlavorName(),
             python_version_str,
             getCommercialVersion() or "not installed",
         )
@@ -969,7 +988,7 @@ def _main():
 
     # Initialize the importing layer from options, main filenames, debugging
     # options, etc.
-    Importing.setupImportingFromOptions()
+    setupImportingFromOptions()
 
     addIncludedDataFilesFromFileOptions()
     addIncludedDataFilesFromPackageOptions()
@@ -996,7 +1015,7 @@ def _main():
         )
 
     # Relaunch in case of Python PGO input to be produced.
-    if Options.shallCreatePgoInput():
+    if Options.shallCreatePythonPgoInput():
         # Will not return.
         pgo_filename = OutputDirectories.getPgoRunInputFilename()
         general.info(
@@ -1036,7 +1055,6 @@ def _main():
     copyDataFiles(standalone_entry_points=getStandaloneEntryPoints())
 
     if Options.isStandaloneMode():
-
         Plugins.onStandaloneDistributionFinished(dist_dir)
 
         if Options.isOnefileMode():
@@ -1045,7 +1063,12 @@ def _main():
             if Options.isRemoveBuildDir():
                 general.info("Removing dist folder '%s'." % dist_dir)
 
-                removeDirectory(path=dist_dir, ignore_errors=False)
+                removeDirectory(
+                    path=dist_dir,
+                    logger=general,
+                    ignore_errors=False,
+                    extra_recommendation=None,
+                )
             else:
                 general.info(
                     "Keeping dist folder '%s' for inspection, no need to use it."
@@ -1062,7 +1085,12 @@ def _main():
         readSconsReport(source_dir)
         readSconsErrorReport(source_dir)
 
-        removeDirectory(path=source_dir, ignore_errors=False)
+        removeDirectory(
+            path=source_dir,
+            logger=general,
+            ignore_errors=False,
+            extra_recommendation=None,
+        )
         assert not os.path.exists(source_dir)
     else:
         general.info("Keeping build directory '%s'." % source_dir)
@@ -1098,10 +1126,11 @@ def _main():
                 """\
 The compilation result is hidden by package directory '%s'. Importing will \
 not use compiled code while it exists."""
-                % base_path
+                % base_path,
+                mnemonic="compiled-package-hidden-by-package",
             )
 
-    general.info("Successfully created '%s'." % final_filename)
+    general.info("Successfully created '%s'." % getReportPath(final_filename))
 
     writeCompilationReports(aborted=False)
 
@@ -1131,6 +1160,8 @@ def main():
     except BaseException:
         try:
             writeCompilationReports(aborted=True)
+        except KeyboardInterrupt:
+            general.warning("""Report writing was prevented by user interrupt.""")
         except BaseException as e:  # Catch all the things, pylint: disable=broad-except
             general.warning(
                 """\
