@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """DLL dependency scan methods for macOS. """
@@ -16,6 +16,7 @@ from nuitka.PythonVersions import python_version
 from nuitka.Tracing import inclusion_logger
 from nuitka.utils.FileOperations import (
     areSamePaths,
+    changeFilenameExtension,
     getReportPath,
     isFilenameBelowPath,
 )
@@ -25,6 +26,7 @@ from nuitka.utils.SharedLibraries import (
     callInstallNameTool,
     getOtoolDependencyOutput,
     getOtoolListing,
+    parseOtoolListingOutput,
 )
 from nuitka.utils.Utils import getArchitecture
 
@@ -55,7 +57,12 @@ def _detectPythonRpaths():
 
 
 def detectBinaryPathDLLsMacOS(
-    original_dir, binary_filename, package_name, keep_unresolved, recursive
+    original_dir,
+    binary_filename,
+    package_name,
+    keep_unresolved,
+    recursive,
+    recursive_dlls=None,
 ):
     assert os.path.exists(binary_filename), binary_filename
 
@@ -89,13 +96,25 @@ def detectBinaryPathDLLsMacOS(
     if recursive:
         merged_result = OrderedDict(resolved_result)
 
+        # For recursive DLL detection, cycle may exist, so we keep track of what
+        # was seen so far.
+        if recursive_dlls is None:
+            recursive_dlls = set([binary_filename])
+        else:
+            recursive_dlls = set(recursive_dlls)
+            recursive_dlls.add(binary_filename)
+
         for sub_dll_filename in resolved_result:
+            if sub_dll_filename in recursive_dlls:
+                continue
+
             _, sub_result = detectBinaryPathDLLsMacOS(
                 original_dir=os.path.dirname(sub_dll_filename),
                 binary_filename=sub_dll_filename,
                 package_name=package_name,
                 keep_unresolved=True,
                 recursive=True,
+                recursive_dlls=recursive_dlls,
             )
 
             merged_result.update(sub_result)
@@ -109,17 +128,9 @@ def detectBinaryPathDLLsMacOS(
 
 
 def _parseOtoolListingOutput(output):
-    paths = OrderedSet()
-
-    for line in output.split(b"\n")[1:]:
-        if str is not bytes:
-            line = line.decode("utf8")
-
-        if not line:
-            continue
-
-        filename = line.split(" (", 1)[0].strip()
+    result = OrderedSet()
 
+    for filename in parseOtoolListingOutput(output):
         # Ignore dependency from system paths.
         if not isFilenameBelowPath(
             path=(
@@ -129,16 +140,19 @@ def _parseOtoolListingOutput(output):
             ),
             filename=filename,
         ):
-            paths.add(filename)
+            result.add(filename)
 
-    return paths
+    return result
 
 
 def _getNonVersionedDllFilenames2(filename):
     yield filename
 
-    if getArchitecture() == "arm64" and filename.endswith(".dylib"):
-        yield filename[:-6] + "_arm64.dylib"
+    if filename.endswith(".dylib"):
+        if getArchitecture() == "arm64":
+            yield filename[:-6] + "_arm64.dylib"
+        else:
+            yield filename[:-6] + "_x86_64.dylib"
 
     match = re.match(r"^(.*?)(\.\d+)+\.dylib$", filename)
 
@@ -150,6 +164,11 @@ def _getNonVersionedDllFilenames2(filename):
         # examples to be sure they are covered with tests.
         if getArchitecture() == "arm64":
             yield match.group(1) + "_arm64.dylib"
+        else:
+            yield match.group(1) + "_x86_64.dylib"
+
+    if filename.endswith(".so"):
+        yield changeFilenameExtension(filename, ".dylib")
 
 
 def _getNonVersionedDllFilenames(dll_filename, package_name):
@@ -264,30 +283,39 @@ def _resolveBinaryPathDLLsMacOS(
                     break
 
         # Sometimes self-dependencies are on a numbered version, but deployed is
-        # one version without it. The macOS just ignores that, and so we do.
+        # one version without it. Also be tolerant about changing suffixes
+        # internally. The macOS just ignores that, and so we do.
         if not os.path.exists(resolved_path):
             if os.path.basename(resolved_path) == os.path.basename(binary_filename):
                 resolved_path = binary_filename
             else:
-                match = re.match(r"^(.*?)(\.\d+)+\.dylib$", resolved_path)
+                for suffix1, suffix2 in (
+                    (".dylib", ".dylib"),
+                    (".so", ".so"),
+                    (".dylib", ".so"),
+                    (".so", ".dylib"),
+                ):
+                    pattern = r"^(.*?)(\.\d+)*%s$" % re.escape(suffix1)
 
-                if match:
-                    candidate = match.group(1) + ".dylib"
+                    match = re.match(pattern, resolved_path)
 
-                    if os.path.exists(candidate):
-                        resolved_path = candidate
-                    else:
-                        candidate = os.path.join(
-                            original_dir, os.path.basename(candidate)
-                        )
+                    if match:
+                        candidate = match.group(1) + suffix2
 
                         if os.path.exists(candidate):
                             resolved_path = candidate
-                        elif os.path.basename(candidate) == os.path.basename(
-                            binary_filename
-                        ):
-                            # Versioned dependency on itself in non-existent path.
-                            resolved_path = binary_filename
+                        else:
+                            candidate = os.path.join(
+                                original_dir, os.path.basename(candidate)
+                            )
+
+                            if os.path.exists(candidate):
+                                resolved_path = candidate
+                            elif os.path.basename(candidate) == os.path.basename(
+                                binary_filename
+                            ):
+                                # Versioned dependency on itself in non-existent path.
+                                resolved_path = binary_filename
 
         if not os.path.exists(resolved_path):
             acceptable, plugin_name = Plugins.isAcceptableMissingDLL(
@@ -331,7 +359,7 @@ def _resolveBinaryPathDLLsMacOS(
 
 
 def _detectBinaryRPathsMacOS(original_dir, binary_filename):
-    stdout = getOtoolListing(binary_filename)
+    stdout = getOtoolListing(filename=binary_filename, cached=True)
 
     lines = stdout.split(b"\n")
 
