@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """ Locating modules and package source on disk.
@@ -36,7 +36,11 @@ from nuitka.PythonVersions import python_version
 from nuitka.Tracing import my_print, recursion_logger
 from nuitka.tree.ReformulationMultidist import locateMultidistModule
 from nuitka.utils.AppDirs import getCacheDir
-from nuitka.utils.FileOperations import listDir, removeDirectory
+from nuitka.utils.FileOperations import (
+    getNormalizedPath,
+    listDir,
+    removeDirectory,
+)
 from nuitka.utils.Hashing import getFileContentsHash
 from nuitka.utils.Importing import (
     builtin_module_names,
@@ -49,7 +53,11 @@ from nuitka.utils.ModuleNames import ModuleName
 from nuitka.utils.SharedLibraries import (
     hasUniversalOrMatchingMacOSArchitecture,
 )
-from nuitka.utils.Utils import isMacOS, isWin32OrPosixWindows
+from nuitka.utils.Utils import (
+    getLaunchingNuitkaProcessEnvironmentValue,
+    isMacOS,
+    isWin32OrPosixWindows,
+)
 
 from .IgnoreListing import isIgnoreListedNotExistingModule
 from .PreloadedPackages import getPreloadedPackagePath, isPreloadedPackagePath
@@ -185,7 +193,7 @@ def hasMainScriptDirectory():
 def isPackageDir(dirname):
     """Decide if a directory is a package.
 
-    Before Python3.3 it's required to have a "__init__.py" file, but then
+    Before Python3 it's required to have a "__init__.py" file, but then
     it became impossible to decide, and for extra fun, there is also the
     extra packages provided via "*.pth" file tricks by "site.py" loading.
     """
@@ -201,6 +209,40 @@ def isPackageDir(dirname):
     )
 
 
+_package_module_name_cache = {}
+
+
+def isPackageModuleName(module_name):
+    """Decide if the give n module name is a package or not.
+
+    module, Cached as it involves disk access."""
+
+    if module_name not in _package_module_name_cache:
+        _module_name, module_filename, _module_kind, _finding = locateModule(
+            module_name=module_name,
+            parent_package=None,
+            level=0,
+        )
+
+        (
+            _main_added,
+            is_package,
+            _is_namespace,
+            _source_ref,
+            _source_filename,
+        ) = decideModuleSourceRef(
+            filename=module_filename,
+            module_name=module_name,
+            is_main=False,
+            is_fake=False,
+            logger=None,
+        )
+
+        _package_module_name_cache[module_name] = is_package
+
+    return _package_module_name_cache[module_name]
+
+
 def getModuleNameAndKindFromFilename(module_filename):
     """Given a filename, decide the module name and kind.
 
@@ -340,7 +382,7 @@ def findModule(module_name, parent_package, level):
         # TODO: Should give a warning and return not found if the levels
         # exceed the package name.
         if parent_package is not None:
-            parent_package = parent_package.getRelativePackageName(level)
+            parent_package = parent_package.getRelativePackageName(level - 1)
         else:
             return None, None, None, "not-found"
 
@@ -431,7 +473,8 @@ ImportScanFinding = collections.namedtuple(
 # We put here things that are not worth it (Cython is not really used by
 # anything really, or where it's know to not have a big # impact, e.g. lxml.
 
-unworthy_namespaces = ("Cython", "lxml")
+# TODO: Do we really want this warning to persist?
+unworthy_namespaces = ("Cython", "lxml", "black", "tomli")
 
 
 def _reportCandidates(package_name, module_name, candidate, candidates):
@@ -642,7 +685,12 @@ def _unpackPathElement(path_entry):
                         zip_ref.extractall(target_dir)
                         zip_ref.close()
                     except BaseException:
-                        removeDirectory(target_dir, ignore_errors=True)
+                        removeDirectory(
+                            target_dir,
+                            logger=recursion_logger,
+                            ignore_errors=True,
+                            extra_recommendation=None,
+                        )
                         raise
 
                 _egg_files[path_entry] = target_dir
@@ -705,8 +753,9 @@ def getPackageSearchPath(package_name):
                 if isPackageDir(package_dir) or force_package:
                     result.append(package_dir)
 
-    result = [element for element in result if os.path.exists(element)]
-    return OrderedSet(result)
+    return OrderedSet(
+        element for element in OrderedSet(result) if os.path.exists(element)
+    )
 
 
 def _findModuleInPath(module_name):
@@ -719,7 +768,7 @@ def _findModuleInPath(module_name):
     # executed, while we normally search in PYTHONPATH after it was executed,
     # and on some systems, that fails.
     if package_name is None and module_name == "site":
-        candidate = os.getenv("NUITKA_SITE_FILENAME")
+        candidate = getLaunchingNuitkaProcessEnvironmentValue("NUITKA_SITE_FILENAME")
 
         if candidate:
             return candidate, "py"
@@ -817,10 +866,10 @@ def locateModule(module_name, parent_package, level):
 
     assert module_package is None or (
         type(module_package) is ModuleName and module_package != ""
-    ), repr(module_package)
+    ), ("Must not attempt to locate %r" % module_name)
 
     if module_filename is not None:
-        module_filename = os.path.normpath(module_filename)
+        module_filename = getNormalizedPath(module_filename)
 
         module_name, module_kind = getModuleNameAndKindFromFilename(module_filename)
         module_name = ModuleName.makeModuleNameInPackage(module_name, module_package)
@@ -865,7 +914,7 @@ def decideModuleSourceRef(filename, module_name, is_main, is_fake, logger):
     is_package = False
 
     if is_main and os.path.isdir(filename):
-        source_filename = os.path.join(filename, "__main__.py")
+        source_filename = getNormalizedPath(os.path.join(filename, "__main__.py"))
 
         if not os.path.isfile(source_filename):
             sys.stderr.write(
@@ -895,7 +944,7 @@ def decideModuleSourceRef(filename, module_name, is_main, is_fake, logger):
     elif isPackageDir(filename):
         is_package = True
 
-        source_filename = os.path.join(filename, "__init__.py")
+        source_filename = getNormalizedPath(os.path.join(filename, "__init__.py"))
 
         if not os.path.isfile(source_filename):
             source_ref = SourceCodeReferences.fromFilename(
@@ -904,7 +953,7 @@ def decideModuleSourceRef(filename, module_name, is_main, is_fake, logger):
             is_namespace = True
         else:
             source_ref = SourceCodeReferences.fromFilename(
-                filename=os.path.abspath(source_filename)
+                filename=getNormalizedPath(os.path.abspath(source_filename))
             )
 
     else:
@@ -1033,6 +1082,7 @@ _stdlib_module_raises = {
     "_interpreters": False,
     "_interpqueues": False,
     "_sysconfig": False,
+    "_suggestions": False,
 }
 
 
@@ -1043,6 +1093,28 @@ def isNonRaisingBuiltinModule(module_name):
     return _stdlib_module_raises.get(module_name)
 
 
+def _getChildPackageNames(module_name):
+    module_name = ModuleName(module_name)
+
+    _module_name, module_filename, _module_kind, _finding = locateModule(
+        parent_package=None, module_name=module_name, level=0
+    )
+
+    package_dir = module_filename
+
+    for module_info in iter_modules([package_dir]):
+        child_name = module_name.getChildNamed(module_info.name)
+        yield child_name
+
+        if module_info.ispkg:
+            for sub_module_name in getChildPackageNames(child_name):
+                yield sub_module_name
+
+
+def getChildPackageNames(module_name):
+    return tuple(_getChildPackageNames(module_name))
+
+
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
