@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """ Test Nuitka compiling itself and compiling itself in compiled form again.
@@ -40,6 +40,7 @@ from nuitka.utils.Execution import wrapCommandForDebuggerForSubprocess
 from nuitka.utils.FileOperations import (
     copyTree,
     deleteFile,
+    getFileContents,
     listDir,
     removeDirectory,
 )
@@ -83,9 +84,9 @@ exe_suffix = ".exe" if os.name == "nt" else ".bin"
 
 def readSource(filename):
     if str is bytes:
-        return open(filename, "rb").read()
+        return getFileContents(filename, mode="rb")
     else:
-        return open(filename, "rb").read().decode("latin1")
+        return getFileContents(filename, encoding="latin1")
 
 
 def diffRecursive(dir1, dir2):
@@ -195,7 +196,12 @@ def executePASS1():
         source_dir = os.path.join(base_dir, package)
         target_dir = package
 
-        removeDirectory(path=target_dir, ignore_errors=False)
+        removeDirectory(
+            path=target_dir,
+            logger=test_logger,
+            ignore_errors=False,
+            extra_recommendation=None,
+        )
 
         os.mkdir(target_dir)
 
@@ -216,7 +222,7 @@ def executePASS1():
                 command = [
                     os.environ["PYTHON"],
                     nuitka_main_path,
-                    "--module",
+                    "--mode=module",
                     "--nofollow-imports",
                     "--output-dir=%s" % target_dir,
                     "--no-pyi-file",
@@ -325,11 +331,16 @@ def compileAndCompareWith(nuitka, pass_number):
 
                 target_dir = os.path.join(tmp_dir, target)
 
-                removeDirectory(path=target_dir, ignore_errors=False)
+                removeDirectory(
+                    logger=test_logger,
+                    path=target_dir,
+                    ignore_errors=False,
+                    extra_recommendation=None,
+                )
 
                 command = [
                     nuitka,
-                    "--module",
+                    "--mode=module",
                     "--enable-plugin=pylint-warnings",
                     "--output-dir=%s" % tmp_dir,
                     "--no-pyi-file",
@@ -343,7 +354,7 @@ def compileAndCompareWith(nuitka, pass_number):
 
                 # In case of segfault or assertion triggered, run in debugger.
                 if exit_nuitka in (-11, -6) and sys.platform != "nt":
-                    command2 = wrapCommandForDebuggerForSubprocess(*command)
+                    command2 = wrapCommandForDebuggerForSubprocess(command=command)
                     subprocess.call(command2)
 
                 if exit_nuitka != 0:
@@ -457,7 +468,7 @@ def executePASS5():
         "--nofollow-import-to=nuitka.build.include",
         "--nofollow-import-to=nuitka.build.static_src",
         "--nofollow-import-to=nuitka.tools",
-        "--module",
+        "--mode=module",
         path,
     ]
 
